﻿using System;
using System.Collections.Generic;
using System.Text;

namespace MvcHelper
{
    public class Pager
    {
        public int PageSize { get; set; }
        public int CurrentPage { get; set; }
        public int RecordCount { get; set; }
        public int PageCount { get; set; }
        public string Url { get; set; }
        public int ExtendPage { get; set; }
        private string PageUnion { get; set; }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pagesize">页面尺寸</param>
        /// <param name="curPage">页码</param>
        /// <param name="recordCount">总数</param>
        /// <param name="url">网址</param>
        /// <param name="extendPage"></param>
        public Pager(int pagesize, int curPage, int recordCount, string url, int extendPage)
        {
            PageUnion = "";
            this.PageSize = pagesize;
            this.CurrentPage = curPage;
            this.RecordCount = recordCount;
            this.Url = System.Text.RegularExpressions.Regex.Replace(url, "&?p=\\d{1,10}", "");
            this.ExtendPage = extendPage;
            PageCount = RecordCount % PageSize == 0 ? RecordCount / PageSize : (RecordCount / PageSize + 1);
        }

        /// <summary>
        /// 获取分页字符串
        /// </summary>
        /// <returns></returns>
        public string GetStrClub()
        {
            if (PageCount == 1 || RecordCount == 0)
            {
                return string.Empty;
            }
            int startPage = 1;
            int endPage = 1;

            string t1 = "<a href=\"" + string.Format(Url, (CurrentPage - 1)) + "\">上一页</a>&nbsp;";
            string t2 = "<a href=\"" + string.Format(Url, (CurrentPage + 1)) + "\">下一页</a>";

            if (PageCount < 1) PageCount = 1;
            if (ExtendPage < 3) ExtendPage = 2;

            if (PageCount > ExtendPage)
            {
                if (CurrentPage - (ExtendPage / 2) > 0)
                {
                    if (CurrentPage + (ExtendPage / 2) < PageCount)
                    {
                        startPage = CurrentPage - (ExtendPage / 2);
                        endPage = startPage + ExtendPage - 1;
                    }
                    else
                    {
                        endPage = PageCount;
                        startPage = endPage - ExtendPage + 1;
                        t2 = "";
                    }
                }
                else
                {
                    endPage = ExtendPage;
                    t1 = "";
                }
            }
            else
            {
                startPage = 1;
                endPage = PageCount;
                t1 = "";
                t2 = "";
            }

            StringBuilder s = new StringBuilder("");

            s.Append(t1);
            for (int i = startPage; i <= endPage; i++)
            {
                if (i == CurrentPage)
                {
                    s.AppendFormat("{0}&nbsp;", i);
                }
                else
                {
                    s.AppendFormat("<a href=\"{0}\">{1}</a>&nbsp;", string.Format(Url, i), i);
                }
            }
            s.Append(t2);

            return s.ToString();
        }
    }
}
