﻿using System;
using System.Collections.Generic;
using System.Text;

namespace MvcHelper
{
    public class Paging<T> where T : class
    {
        public Pager Pager;

        /// <summary>
        /// 
        /// </summary>
        public Paging()
        {
        }
        /// <summary>
        /// 初始化
        /// </summary>
        /// <param name="pageSize">页面尺寸</param>
        /// <param name="currentPageIndex">页码</param>
        /// <param name="dataSource">数据源</param>
        /// <param name="totalCount">记录总数</param>
        /// <param name="url">分页参数 如/Blog/A/{0}</param>
        public Paging(int pageSize, int currentPageIndex, List<T> dataSource, int totalCount, string url)
        {
            this.PageSize = pageSize;
            this.CurrntPageIndex = currentPageIndex;
            this.DataList = dataSource;
            this.TotalCount = totalCount;
            Pager = new Pager(pageSize, currentPageIndex, totalCount, url, 8);
        }

        /// <summary>
        /// 数据列表
        /// </summary>
        public List<T> DataList { get; set; }

        int _pageSize;

        /// <summary>
        /// 每页数据条数
        /// </summary>
        public int PageSize
        {
            get
            {
                return _pageSize;
            }
            set
            {
                if (value < 1) throw new ArgumentOutOfRangeException();
                _pageSize = value;

                CalcPage();
            }
        }

        /// <summary>
        /// 总页数
        /// </summary>
        public int PageCount { get; set; }

        int _totalCount;

        /// <summary>
        /// 总条数
        /// </summary>
        public int TotalCount
        {
            get
            {
                return _totalCount;
            }
            set
            {
                _totalCount = value;
                CalcPage();
            }
        }

        int _currentPageIndex;

        /// <summary>
        /// 当前页码
        /// </summary>
        public int CurrntPageIndex
        {
            get
            {
                if (PageSize == 0 || PageCount == 0) return _currentPageIndex;

                if (_currentPageIndex < 1) return 1;
                else if (_currentPageIndex > PageCount) return PageCount;
                else return _currentPageIndex;
            }
            set
            {
                _currentPageIndex = value;

                if (_currentPageIndex < 1) _currentPageIndex = 1;
            }
        }
        /// <summary>
        /// 获取分页数据
        /// </summary>
        /// <returns></returns>
        public string GetStr()
        {
            return Pager.GetStrClub();
        }
        /// <summary>
        /// 重新计算当前页面总数
        /// </summary>
        void CalcPage()
        {
            if (_pageSize == 0) return;

            PageCount = (int)Math.Ceiling(TotalCount * 1.0 / PageSize);
        }

        /// <summary>
        /// 返回当前页的记录起始索引
        /// </summary>
        public int StartIndex
        {
            get
            {
                return (_currentPageIndex - 1) * PageSize;
            }
        }
    }
}
