﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Text.RegularExpressions;
using System.Collections.Specialized;
using System.Threading;
using System.Linq;

namespace MvcHelper
{
    public abstract class ParamBase
    {
      
        /// <summary>
        /// 接受的参数列表，小写，为空或长度为零是无限制
        /// </summary>
        public List<KVP> ParaList { get; set; }
        /// <summary>
        /// 参数化枚举的限制
        /// </summary>
        public Dictionary<string, List<KVP>> ParaStringEnum { get; set; }

        /// <summary>
        /// 正则表达式验证
        /// </summary>
        public Dictionary<string, Regex> ParaRegexValidate { get; set; }


        public NameValueCollection QueryString { get; private set; }
        /// <summary>
        /// 参数列表
        /// </summary>
        public Dictionary<string, string> Params { get; set; }
        public ParamBase()
        {
            ParaList = new List<KVP>();
            Params = new Dictionary<string, string>();
            ParaStringEnum = new Dictionary<string, List<KVP>>();
            ParaRegexValidate = new Dictionary<string, Regex>();
        }
        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="para"></param>
        public void Parse(string para, NameValueCollection query)
        {
            QueryString = query;

            if (string.IsNullOrEmpty(para)) return;
            string[] args = para.Split('_');
            foreach (var argName in args)
            {
                string[] arg = argName.Split('-');
                if (arg.Length != 2 || string.IsNullOrEmpty(arg[0])) continue;

                string n = arg[0].ToLower();
                string v = arg[1].ToLower();

                if ((ParaStringEnum != null && ParaStringEnum.ContainsKey(n)) && ParaStringEnum[n].FindIndex(m => m.Key == v) == -1) continue;
                if ((ParaRegexValidate != null && ParaRegexValidate.ContainsKey(n) && !ParaRegexValidate[n].IsMatch(v))) continue;

                this[n] = v;
            }
        }

        /// <summary>
        /// 获得或设置对应参数
        /// </summary>
        /// <param name="paraName"></param>
        /// <returns></returns>
        public string this[string paraName]
        {
            get
            {
                if (Params.ContainsKey(paraName)) return Params[paraName];
                else return string.Empty;
            }
            set
            {
                paraName = paraName.ToLower();
                string v = value.ToLower();

                if (Params.ContainsKey(paraName)) Params[paraName] = v;
                else
                {
                    if (ParaList != null || ParaList.Count == 0 || ParaList.FindIndex(m => m.Key == paraName) == -1)
                    {
                        Params.Add(paraName, v);
                    }
                }
            }
        }

        /// <summary>
        /// 分页的基础路径
        /// </summary>
        public string BaseUrl { get; set; }
        /// <summary>
        /// 获得路径表达式
        /// </summary>
        /// <param name="ignoreItems"></param>
        /// <returns></returns>
        public string GetParaUrl(params string[] ignoreItems)
        {
            string p = string.Empty;

            foreach (var k in Params.Keys)
            {
                if (ignoreItems.Contains<string>(k)) continue;

                p += k + "-" + this[k] + "_";
            }
            return p.TrimEnd('_');
        }
        /// <summary>
        /// 分页参数
        /// </summary>
        public string PagingUrl
        {
            get
            {
                string baseUrl = BaseUrl;
                string purl = GetParaUrl("page");
                if (!string.IsNullOrEmpty(purl)) purl += "_";
                baseUrl += purl + "page-{0}";

                string query = "";
                if (QueryString != null && QueryString.Count > 0)
                {
                    query = "?";
                    foreach (string item in QueryString.Keys)
                    {
                        query += item + "=" + HttpUtility.UrlEncode(QueryString[item]);
                    }
                }

                if (!string.IsNullOrEmpty(query)) baseUrl += "?" + query;

                return baseUrl;
            }
        }
    }

    /// <summary>
    /// 键值属性类
    /// </summary>
    public class KVP
    {
        /// <summary>
        /// 参数名称
        /// </summary>
        public string Key { get; set; }
        /// <summary>
        /// 参数值
        /// </summary>
        public string Value { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public bool Flag { get; set; }
        /// <summary>
        /// 
        /// </summary>
        public int State { get; set; }

        public KVP(string _k, string _v)
        {
            Key = _k;
            Value = _v;
            Flag = false;
        }
    }
}
